/*
 * LMDB Abstraction Layer.
 * Copyright (C) 2023  Yury Gubich <blue@macaw.me>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#pragma once

#include <QString>
#include <QByteArray>

namespace LMDBAL {

template<>
class Serializer<QString> {
public:
    Serializer():value() {};
    Serializer(const QString& p_value):value(p_value.toUtf8()) {};
    ~Serializer() {};

    QString deserialize(const MDB_val& data) {
#if (QT_VERSION >= QT_VERSION_CHECK(6, 0, 0))
        if (data.mv_size > static_cast<size_t>(std::numeric_limits<qsizetype>::max()))
            throw std::runtime_error("Data size exceeds QByteArray capacity");

        value = QByteArray(static_cast<char *>(data.mv_data), static_cast<qsizetype>(data.mv_size));
#else
        if (data.mv_size > static_cast<size_t>(std::numeric_limits<int>::max()))
            throw std::runtime_error("Data size exceeds QByteArray capacity");

        value = QByteArray(static_cast<char *>(data.mv_data), static_cast<int>(data.mv_size));
#endif

        return QString::fromUtf8(value);
    };
    void deserialize(const MDB_val& data, QString& result) {
#if (QT_VERSION >= QT_VERSION_CHECK(6, 0, 0))
        if (data.mv_size > static_cast<size_t>(std::numeric_limits<qsizetype>::max()))
            throw std::runtime_error("Data size exceeds QByteArray capacity");

        value = QByteArray(static_cast<char *>(data.mv_data), static_cast<qsizetype>(data.mv_size));
#else
        if (data.mv_size > static_cast<size_t>(std::numeric_limits<int>::max()))
            throw std::runtime_error("Data size exceeds QByteArray capacity");

        value = QByteArray(static_cast<char *>(data.mv_data), static_cast<int>(data.mv_size));
#endif

        result = QString::fromUtf8(value);
    }
    MDB_val setData(const QString& data) {
        value = data.toUtf8();
        return getData();
    };
    MDB_val getData() {
        MDB_val result;
        result.mv_data = value.data();
        result.mv_size = static_cast<size_t>(value.size());
        return result;
    };
    void clear() {};    //not possible;

private:
    QByteArray value;
};

}